import 'package:vector_math/vector_math.dart';
import 'voxel_walker.dart';

abstract class i_surface {
  bool is_cold(Vector3 v);
}

void rotate(Vector3 v) {
  double tmp = v.x;
  v.x = v.y;
  v.y = v.z;
  v.z = tmp;
}

class cube_march_quad {
  final List<Vector3> _vertices = new List<Vector3>(4);
  final List<Vector3> _vertices_reversed = new List<Vector3>(4);
  final double _surface_temperature;
  cube_march_quad(double surface_temperature) : _surface_temperature = surface_temperature {
    _vertices[0] = _vertices_reversed[3] = new Vector3(0.0, 0.0, 0.0);
    _vertices[1] = _vertices_reversed[2] = new Vector3(1.0, 0.0, 0.0);
    _vertices[2] = _vertices_reversed[1] = new Vector3(1.0, 1.0, 0.0);
    _vertices[3] = _vertices_reversed[0] = new Vector3(0.0, 1.0, 0.0);
  }
  static const int _cube_width = 2;
  bool is_cube_on_surface(voxel_walker voxel_walker) {
    int cold_count = 0;
    for(int x = 0; x < _cube_width; x++)
      for(int y = 0; y < _cube_width; y++)
        for(int z = 0; z < _cube_width; z++) {
          double temperature = voxel_walker.get_relative(x, y, z);
          if(temperature < _surface_temperature)
            cold_count++;
        }
    bool on_surface = ((cold_count > 0) && (cold_count < 8));
    return on_surface;
  }
  bool _flipped = false;
  void _flip() {
    _flipped = !_flipped;
    if (_flipped) {
      _origin.setValues(1.0, 1.0, 1.0);
    } else {
      _origin.setValues(0.0, 0.0, 0.0);
    }
    _vertices.forEach((v) => v.scale(-1.0));
  }

  void _rotate_quad_coords() {
    _vertices.forEach((v) {
      rotate(v);
    });
  }

  final Vector3 _origin = new Vector3.zero();

  bool _is_cold_quad(voxel_walker voxel_walker_) {
    int count = 0;
    Iterable<Vector3> vertices = _flipped ? _vertices_reversed : _vertices;
    for(var vert in vertices) {
      vert.copyInto(iter_position);
      iter_position.add(_origin);
      double temperature = voxel_walker_.get_relative(
          iter_position.x.toInt(), iter_position.y.toInt(), iter_position.z.toInt());
      if (temperature < _surface_temperature) count++;
    };
    return (4 == count);
  }

  static final Vector3 iter_position = new Vector3.zero();

  Iterable<Vector3> _append_quad() sync* {
    Iterable<Vector3> vertices = _flipped ? _vertices_reversed : _vertices;
    for (var vert in vertices) {
      vert.copyInto(iter_position);
      iter_position.add(_origin);
      yield iter_position;
    }
  }

  Iterable<Vector3> get_surface_quads(voxel_walker voxel_walker_) sync* {
    //todo, Vector3 heap
    if(is_cube_on_surface(voxel_walker_)) {
      for (int j = 0; j < 3; j++) {
        if (_is_cold_quad(voxel_walker_)) yield* _append_quad();
        _rotate_quad_coords();
      }
      _flip();

      for (int j = 0; j < 3; j++) {
        if (_is_cold_quad(voxel_walker_)) yield* _append_quad();
        _rotate_quad_coords();
      }
      _flip();
    }
  }
}
