import 'dart:math' as math;
import 'package:vector_math/vector_math.dart';
import 'dart:typed_data';
import 'voxel_walker.dart';

class voxel_array {
  final Vector3 _base = new Vector3.zero();
  final int _width;
  final Float32List _voxels;
  final double _scale;
  final int _count;
  voxel_array(Vector3 base, int width)
      : _width = width,
        _count = width - 1,
        _scale = 1.0 / (width - 1),
        _voxels = new Float32List(width * width * width) {
    base.copyInto(_base);
  }
  get base {
    return _base;
  }

  get width {
    return _width;
  }

  get scale {
    return _scale;
  }

  int _offset(int x, int y, int z) {
    assert(x < _width);
    assert(y < _width);
    assert(z < _width);
    return (z * _width * _width) + (y * _width) + x;
  }

  void set(int x, int y, int z, double val) {
    int index = _offset(x, y, z);
    _voxels[index] = val;
  }

  double get(int x, int y, int z) {
    int index = _offset(x, y, z);
    return _voxels[index];
  }

  void get_world_pos(int x, int y, int z, Vector3 out) {
    this._base.copyInto(out);
    out.x += x * _scale;
    out.y += y * _scale;
    out.z += z * _scale;
  }

  static final Vector3 _bounding_sphere_center_calc = new Vector3.zero();
  static final Vector3 _bounding_sphere_radius_calc = new Vector3.zero();
  void bounding_sphere(Sphere out) {
    _base.copyInto(_bounding_sphere_center_calc);
    _bounding_sphere_center_calc.x += ((_width - 1) * _scale / 2.0);
    _bounding_sphere_center_calc.y += ((_width - 1) * _scale / 2.0);
    _bounding_sphere_center_calc.z += ((_width - 1) * _scale / 2.0);
    _bounding_sphere_radius_calc.add(_base);
    _bounding_sphere_center_calc.copyInto(out.center);
    double len = (_width - 1) * _scale / 2.0;
    len = math.sqrt((len * len) + (len * len) + (len * len));
    out.radius = len;
  }

  String toString() {
    return _base.toString();
  }

  static final voxel_walker _walker = new voxel_walker();

  Iterable<voxel_walker> iterator_voxels(bool include_edge) sync* {
    final limit = include_edge ? _width : _count;
    for (int x = 0; x < limit; x++)
      for (int y = 0; y < limit; y++)
        for (int z = 0; z < limit; z++) {
      _walker.x = x;
      _walker.y = y;
      _walker.z = z;
      _walker.voxels = this;
      yield _walker;
    }
  }
}
