import 'package:vector_math/vector_math.dart';
import 'voxel_array.dart';
import 'voxel_walker.dart';

class voxel_db {
  final int _voxel_slices;
  voxel_db(int voxel_slices) : _voxel_slices = voxel_slices {}

  final Map<String, voxel_array> _db = new Map<String, voxel_array>();

  Iterable<voxel_walker> iterator_voxels(bool include_edge) sync* {
    for (voxel_array voxel_array_ in _db.values) {
      yield* voxel_array_.iterator_voxels(include_edge);
    }
  }

  voxel_array get(int x, int y, int z) {
    var v = new Vector3(x.toDouble(), y.toDouble(), z.toDouble());
    var key = v.toString();
    if (_db.containsKey(key)) {
      return _db[key];
    } else {
      var voxel_array_ = new voxel_array(v, _voxel_slices);
      _db[key] = voxel_array_;
      //print("voxel_array_get: ${voxel_array_._base.toString()}");
      return voxel_array_;
    }
  }

  voxel_array get_filled(int x, int y, int z, double val) {
    var voxel_array_ = get(x, y, z);
    voxel_array_.iterator_voxels(true).forEach((voxel_walker) {
      voxel_walker.set(val);
    });
    return voxel_array_;
  }

  void fill_cube(int x0, int y0, int z0, int width, double val) {
    for (int x = x0; x < (x0 + width); x++)
      for (int y = y0; y < (y0 + width); y++)
        for (int z = z0; z < (z0 + width); z++) get_filled(x, y, z, val);
  }
}
