import 'package:vector_math/vector_math.dart';

class voxel_quad {
  static const int _count = 4;
  final positions = new List<Vector3>(_count);
  final uvs = new List<Vector2>(_count);
  final int normal_index_modulo;
  final double position_to_uv_scale;

  voxel_quad(int normal_index_modulo)
      : normal_index_modulo = normal_index_modulo,
        position_to_uv_scale = 1.0 / normal_index_modulo {
    for (int i = 0; i < _count; i++) {
      positions[i] = new Vector3.zero();
      uvs[i] = new Vector2.zero();
    }
  }

  final _normal = new Vector3.zero();
  final _normal_x = new Vector3.zero();
  final _normal_y = new Vector3.zero();

  Vector3 get normal {
    positions[1].copyInto(_normal_x);
    _normal_x.sub(positions[0]);
    positions[3].copyInto(_normal_y);
    _normal_y.sub(positions[0]);
    _normal_x.crossInto(_normal_y, _normal);
    normal.normalize();
    return _normal;
  }

  int get normal_xyz_index {
    var n = normal;
    int index = -1;
    for (int i = 0; i < 3; i++)
      if (n[i] != 0.0)
        index = i;
    else
      assert(n[i] == 0.0);
    assert(index >= 0);
    return index;
  }

  int get texture_index {
    int xyz_index = normal_xyz_index;
    double val = positions[0][xyz_index];
    return (val.toInt() % normal_index_modulo);
  }

  void update_uvs() {
    int normal_xyz_index_snap = normal_xyz_index;
    for (int i = 0; i < _count; i++) {
      for (int uv_index = 0; uv_index < 2; uv_index++) {
        int modulo_xyz_index = (normal_xyz_index_snap + 1 + uv_index) % 3;
        uvs[i][uv_index] = positions[i][modulo_xyz_index] * position_to_uv_scale;
        uv_index++;
      }
    }
  }

  String get texture_bucket_key {
    // normal:normal_modulo_index
    String axis_tag = "axis_";
    var normal_ = this.normal;
    double direction = 0.0;
    if(normal_[0] != 0.0) {
      axis_tag += "x_";
      direction = normal_[0];
    } else if(normal_[1] != 0.0) {
      axis_tag += "y_";
      direction = normal_[1];
    } else if(normal_[2] != 0.0) {
      axis_tag += "z_";
      direction = normal_[2];
    }
    axis_tag += (direction < 0.0) ? "neg_" : "pos_";
    axis_tag += texture_index.toString();
    return axis_tag;
  }

  bool load(Iterable<Vector3> quads) {
    final iterator = quads.iterator;
    for (int i = 0; i < _count; i++) {
      if(!iterator.moveNext())
        return false;
      iterator.current.copyInto(positions[i]);
    }
    update_uvs();
    return true;
  }
}
