import 'package:rad_textures/rad_textures.dart' as rad;
import 'package:vector_math/vector_math.dart';

abstract class runner {
  void init();
  void run();
}

class test_voxel_array_1 implements runner {
  static const _width = 4;
  final _voxel_array = new rad.voxel_array(new Vector3.zero(), _width);
  init() {}

  @override
  void run() {
    for(int x = 0; x < _width; x++)
      for(int y = 0; y < _width; y++)
        for(int z = 0; z < _width; z++) {
          int val = (z * _width * _width) +
              (y * _width * _width) +
              x;
          _voxel_array.set(x, y, z, val.toDouble());
        }
    for(int x = 0; x < _width; x++)
      for(int y = 0; y < _width; y++)
        for(int z = 0; z < _width; z++) {
          int val = (z * _width * _width) +
              (y * _width * _width) +
              x;
          double check = _voxel_array.get(x, y, z);
          assert(val == check);
        }
    print("width: ${_width}");
    int count = 0;
    _voxel_array.iterator_voxels(true).forEach((voxel_walker) {
      count++;
    });
    print("width^3: ${count}");
    assert(count == (_width * _width * _width));
  }
}

class test_voxel_db implements runner {
  final _voxel_db = new rad.voxel_db(5);
  static const int _cube_width = 2;
  static const double _radius = 2.0;
  double min_radius = 100.0;
  double max_radius = 0.0;
  init() {
    _voxel_db.fill_cube(
        -_cube_width, -_cube_width, -_cube_width, 2 * _cube_width, 0.0);
    int voxel_count = 0;
    int in_sphere_count = 0;
    _voxel_db.iterator_voxels(true).forEach((voxel_walker) {
      if (voxel_walker.world_position.length < _radius) {
        voxel_walker.set(1.0);
        in_sphere_count++;
      }
      voxel_count++;
    });
    print("voxel_count: ${voxel_count}, in_sphere_count: ${in_sphere_count}");
  }

  @override
  void run() {
    var quad_extractor = new rad.voxel_quad_extractor(_voxel_db, 0.5);
    int count = 0;
    double sum = 0.0;
    final centroid = new Vector3.zero();
    quad_extractor.iterate_quads().forEach((v) {
      count++;
      centroid.add(v);
      sum += v.length;
      if(v.length > max_radius) max_radius = v.length;
      if(v.length < min_radius) min_radius = v.length;
      //print("v.length: ${v.length}");
    });
    sum /= count.toDouble();
    centroid.scale(1.0 / count.toDouble());
    print("average radius: ${sum}, quad_vertex_count: ${count}, centroid: ${centroid.toString()}");
    print("min_radius: ${min_radius}");
    print("max_radius: ${max_radius}");
  }
}

void main(List<String> args) {
  print("start_tests");
  List<runner> queue = [
    new test_voxel_array_1(),
    new test_voxel_db()
  ];
  queue.forEach((runner) {
    print("---");
    print(runner.runtimeType.toString());
    runner.init();
    runner.run();
  });
  print("finished_tests");
}
