import 'dart:html';
import 'package:rad_textures/rad_textures.dart';
import 'package:vector_math/vector_math.dart';

void main() {
  print('starting...');
  _create();
  print('starting2b...');
}

CanvasElement _append_canvas(String label) {
  DivElement div = document.getElementById("noises_pane");
  CanvasElement canvas = document.createElement("canvas");
  canvas.width = 512;
  canvas.height = canvas.width;
  div.append(canvas);
  div.append(document.createElement("br"));
  SpanElement span = document.createElement("span");
  span.innerHtml = label;
  div.append(span);
  div.append(document.createElement("br"));
  return canvas;
}

double scale = 0.03;

void _plot_bumps(String canvas_id, TSampler noise_callback) {
  CanvasElement canvas = _append_canvas(canvas_id);
  CanvasRenderingContext2D ctx = canvas.getContext('2d');

  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  Vector3 vec_00 = new Vector3.zero();
  Vector3 vec_10 = new Vector3.zero();
  Vector3 vec_01 = new Vector3.zero();
  Vector3 normal = new Vector3.zero();
  for (int x = 0; x < canvas.width; x++) {
    for (int y = 0; y < canvas.height; y++) {
      double sample_00 = noise_callback(new Vector3(x * scale, y * scale, 0.0));
      double sample_10 =
          noise_callback(new Vector3((x + 1) * scale, y * scale, 0.0));
      double sample_01 =
          noise_callback(new Vector3(x * scale, (y + 1) * scale, 0.0));
      vec_00.setValues(x * scale, y * scale, sample_00);
      vec_10.setValues((x + 1) * scale, y * scale, sample_10);
      vec_01.setValues(x * scale, (y + 1) * scale, sample_01);
      vec_10.sub(vec_00);
      vec_10.normalize();
      vec_01.sub(vec_00);
      vec_01.normalize();
      vec_10.crossInto(vec_01, normal);
      normal.normalize();
      imageData.data[(y * canvas.width + x) * 4 + 0] =
          map_double_to_int(normal.x);
      imageData.data[(y * canvas.width + x) * 4 + 1] =
          map_double_to_int(normal.y);
      imageData.data[(y * canvas.width + x) * 4 + 2] =
          map_double_to_int(normal.z);
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}

void _plot_noise(String canvas_id, TSampler noise_callback) {
  CanvasElement canvas = _append_canvas(canvas_id);
  CanvasRenderingContext2D ctx = canvas.getContext('2d');
  ;
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  for (int x = 0; x < canvas.width; x++) {
    for (int y = 0; y < canvas.height; y++) {
      double sample = noise_callback(new Vector3(x * scale, y * scale, 0.0));
      imageData.data[(y * canvas.width + x) * 4 + 0] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 1] =
          FASTFLOOR((sample * 0.5 + 0.5) * 255);
      imageData.data[(y * canvas.width + x) * 4 + 2] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}

void _create() {
  print('creating...');
  _plot_noise("canvas_periodic_noise", pnoise3_map);
  //_plot_bumps("canvas_periodic_noise_bumps", pnoise3_map);
  _plot_noise("canvas_simplex_noise", noise3_map);
  _plot_bumps("canvas_simplex_noise_bumps", noise3_map);
  _plot_noise("canvas_ridged_multifractal", RidgedMultifractal_Default_map);
  //_plot_bumps("canvas_ridged_multifractal_bumps", RidgedMultifractal_Default_map);
  _plot_noise("canvas_turbulence", turbulence_default_map);
  //_plot_bumps("canvas_turbulence_bumps", turbulence_default_map);
  _plot_noise("canvas_cell_noise", FCellNoise3_map);
  _plot_noise("canvas_fbm", fBm_default_map);
  //_plot_bumps("canvas_fbm_bumps", fBm_default_map);
  _plot_noise("canvas_hybrid_multifractal", HybridMultifractal_map);
  //_plot_bumps("canvas_hybrid_multifractal_bumps", HybridMultifractal_map);
  _plot_noise("worley", worley_map);
}
