import 'dart:html';
import 'package:rad_textures/rad_textures.dart';
import 'package:vector_math/vector_math.dart';

void main() {
    _create_sphere();
    var url = _create_normals();
    ImageElement img = querySelector("#canvas_ray_traced_normals_image");
    img.src = url;
}

void _create_sphere() {
  CanvasElement canvas = querySelector("#canvas_ray_traced_sphere");
  CanvasRenderingContext2D ctx = canvas.context2D;
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  Vector3 vSphere = new Vector3(0.0, 0.0, 5.0);
  Vector3 vCamera = new Vector3(0.0, 0.0, -6.0);
  int half_canvas_width = canvas.width ~/ 2;
  int half_canvas_height = canvas.height ~/ 2;
  for (int x = 0; x < canvas.width; x++) {
    for (int y = 0; y < canvas.height; y++) {
      Vector3 vScreen = new Vector3((x - half_canvas_width) / 100.0,
          (y - half_canvas_height) / 100.0, 0.0);
      imageData.data[(y * canvas.width + x) * 4 + 0] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 1] =
          (SphereRay(vCamera, vSphere, vScreen) * 200).toInt();
      imageData.data[(y * canvas.width + x) * 4 + 2] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}

String _create_normals() {
  CanvasElement canvas = querySelector("#canvas_ray_traced_normals");
  CanvasRenderingContext2D ctx = canvas.context2D;
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  Vector3 vSphere = new Vector3(0.0, 0.0, 5.0);
  Vector3 vCamera = new Vector3(0.0, 0.0, -6.0);
  int half_canvas_width = canvas.width ~/ 2;
  int half_canvas_height = canvas.height ~/ 2;
  for (int x = 0; x < canvas.width; x++) {
    for (int y = 0; y < canvas.height; y++) {
      Vector3 vScreen = new Vector3((x - half_canvas_width) / 100.0,
          (y - half_canvas_height) / 100.0, 0.0);
      var n = SphereRayNormal(vCamera, vSphere, vScreen);
      imageData.data[(y * canvas.width + x) * 4 + 0] = mapRange255(n.x);
      imageData.data[(y * canvas.width + x) * 4 + 1] = mapRange255(-n.y);
      imageData.data[(y * canvas.width + x) * 4 + 2] = mapRange255(-n.z);
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
  return canvas.toDataUrl();
}
